{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans %}Create a job template with a specified name.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Select the type of your job:{% endblocktrans %}
  <ul class="list-bullet">
    <li>{% blocktrans %}Pig{% endblocktrans %}</li>
    <li>{% blocktrans %}Hive{% endblocktrans %}</li>
    <li>{% blocktrans %}Spark{% endblocktrans %}</li>
    <li>{% blocktrans %}Storm{% endblocktrans %}</li>
    <li>{% blocktrans %}Storm Pyleus{% endblocktrans %}</li>
    <li>{% blocktrans %}MapReduce{% endblocktrans %}</li>
    <li>{% blocktrans %}Java Action{% endblocktrans %}</li>
    <li>{% blocktrans %}Shell Action{% endblocktrans %}</li>
  </ul>
</p>
<p>
  {% blocktrans %}Choose or create your main binary.  Additional libraries can be added from the "Libs" tab.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}For Storm, Storm Pyleus, Spark and Shell jobs, only a main is required,"libs" are optional.{% endblocktrans %}
</p>
<p>
  {% blocktrans trimmed %}For MapReduce or Java Action jobs, "mains" are not applicable.  You are required to add one
  or more "libs" for these jobs.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}You may also enter an optional description for your job template.{% endblocktrans %}
</p>
</div>
